﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ObjectSelectionBehaviour : MonoBehaviour
{
    public float menuOffset;
    public float slideSpeed;
    public float buttonSpeed;

    private float slideDistance = 0;
    private bool slide;
    private RectTransform header;
    private RectTransform slidingBg;
    private Transform objectButtonGroup;
    private bool buttonsEnabled;

    private void Start()
    {
        // Get the header, slidingBg and objectButtonGroup by child index
        header = transform.GetChild(0).GetComponent<RectTransform>();
        slidingBg = transform.GetChild(1).GetComponent<RectTransform>();
        objectButtonGroup = transform.GetChild(3);

        // Set the scale of all buttons to 0
        objectButtonGroup.localScale = new Vector3(0, 1, 1);

        // Calculate the slide distance
        foreach (Transform child in objectButtonGroup)
            if (child.localPosition.y > slideDistance)
                slideDistance = child.localPosition.y;
        slideDistance += menuOffset;
    }

    private void Update()
    {
        // If object starts sliding and hasn't reached target yet
        if (slide && slidingBg.sizeDelta.y < slideDistance)
        {
            // Slide background
            slidingBg.sizeDelta = new Vector2(slidingBg.sizeDelta.x, slidingBg.sizeDelta.y + slideSpeed * Time.deltaTime);

            // Snap objects to the right location
            if (slidingBg.sizeDelta.y > slideDistance)
                slidingBg.sizeDelta = new Vector2(slidingBg.sizeDelta.x, slideDistance);

            // Make header follow
            header.localPosition = new Vector3(header.localPosition.x, slidingBg.sizeDelta.y, header.localPosition.z);
        }
        else if (!slide && slidingBg.sizeDelta.y > 0 && objectButtonGroup.localScale.x <= 0)
        {
            // Slide background
            slidingBg.sizeDelta = new Vector2(slidingBg.sizeDelta.x, slidingBg.sizeDelta.y - slideSpeed * Time.deltaTime);

            // Snap objects to the right location
            if (slidingBg.sizeDelta.y < 0)
                slidingBg.sizeDelta = new Vector2(slidingBg.sizeDelta.x, 0);

            // Make header follow
            header.localPosition = new Vector3(header.localPosition.x, slidingBg.sizeDelta.y, header.localPosition.z);
        }
        else if (slide && !buttonsEnabled)
        {
            // Enable object buttons
            foreach (Transform child in objectButtonGroup)
                child.gameObject.SetActive(true);

            buttonsEnabled = true;
        }
        else if (!slide && objectButtonGroup.localScale.x > 0)
        {
            objectButtonGroup.localScale = new Vector3(objectButtonGroup.localScale.x - buttonSpeed * Time.deltaTime, 1, 1);

            // Snap buttons to right size
            if (objectButtonGroup.localScale.x < 0)
                objectButtonGroup.localScale = new Vector3(0, 1, 1);

            // Disable buttons if x scale is 0
            if (objectButtonGroup.localScale.x == 0)
                objectButtonGroup.gameObject.SetActive(false);
        }

        // If buttons are enabled and haven't been scaled up yet
        if (buttonsEnabled && objectButtonGroup.localScale.x < 1)
        {
            // Enable button group if not done yet
            if (!objectButtonGroup.gameObject.activeSelf)
                objectButtonGroup.gameObject.SetActive(true);

            // Scale up buttons
            objectButtonGroup.localScale = new Vector3(objectButtonGroup.localScale.x + buttonSpeed * Time.deltaTime, 1, 1);

            // Snap buttons to right size
            if (objectButtonGroup.localScale.x > 1)
                objectButtonGroup.localScale = new Vector3(1, 1, 1);
        }
    }

    /// <summary>
    /// Slides the menu in and out
    /// </summary>
    public void Slide()
    {
        slide = !slide;

        if (!slide)
            buttonsEnabled = false;
    }
}
