﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlacementProperties : MonoBehaviour {
    [Header("Images")]
    public Sprite unconnected;
    public Sprite connectedBothSides;
    public Sprite connectedOneSide;
    public Sprite corner;
    public Sprite tJunction;
    public Sprite crossing;
    [Space(10)]

    [Header("Properties")]
    public float strength;
    public enum ObjectTypes {Blockade, Divider, Drainage};
    public ObjectTypes type;
    public float health;
    public bool connectToOthers;
    public bool canBePlacedOnSand = true;
    public bool canBePlacedOnGrass = true;
    public string objectName;
    public string description;
    public bool destroyed = false;
    public int cost;
    [Space(10)]

    public bool updateOthers = true;

    // Tiles
    private Transform upTile;
    private Transform downTile;
    private Transform leftTile;
    private Transform rightTile;

    private SpriteRenderer spriteRenderer;

    private void Start()
    {
        // Get hashtable from parent
        Hashtable tiles = GetComponentInParent<TileCollector>().tiles;

        // Get adjacent tile positions
        Vector2 upTilePosition = new Vector2(transform.position.x, transform.position.y + 1);
        Vector2 downTilePosition = new Vector2(transform.position.x, transform.position.y - 1);
        Vector2 leftTilePosition = new Vector2(transform.position.x - 1, transform.position.y);
        Vector2 rightTilePosition = new Vector2(transform.position.x + 1, transform.position.y);

        // Get adjacent tiles
        if (tiles.Contains(upTilePosition))
            upTile = (Transform) tiles[upTilePosition];
        if (tiles.Contains(downTilePosition))
            downTile = (Transform) tiles[downTilePosition];
        if (tiles.Contains(leftTilePosition))
            leftTile = (Transform) tiles[leftTilePosition];
        if (tiles.Contains(rightTilePosition))
            rightTile = (Transform) tiles[rightTilePosition];

        // Get sprite renderer
        spriteRenderer = GetComponent<SpriteRenderer>();

        // Update the orientation
        if (connectToOthers)
            UpdateOrientation(updateOthers);
    }

    private void FixedUpdate()
    {
        //fade object away, then actually destroy it.
        if (destroyed)
        {
            Color c = spriteRenderer.color;
            c.a -= 0.01f;
            spriteRenderer.color = c;
            if (c.a <= 0)
            {
               GameObject.Destroy(gameObject);
            }
        }
    }

    /// <summary>
    /// Updates the orientation of a placeable object
    /// </summary>
    public void UpdateOrientation(bool updateConnected)
    {
        bool upTileConnected = false;
        bool downTileConnected = false;
        bool leftTileConnected = false;
        bool rightTileConnected = false;

        // Check if object is connected to same object for every side
        // If object is connected, update the connected object
        if (upTile != null && upTile.childCount > 0 && upTile.GetChild(0).GetComponent<PlacementProperties>().objectName == objectName)
        {
            upTileConnected = true;

            if (updateConnected)
                upTile.GetChild(0).GetComponent<PlacementProperties>().UpdateOrientation(false);
        }
        if (downTile != null && downTile.childCount > 0 && downTile.GetChild(0).GetComponent<PlacementProperties>().objectName == objectName)
        {
            downTileConnected = true;

            if (updateConnected)
                downTile.GetChild(0).GetComponent<PlacementProperties>().UpdateOrientation(false);
        }
        if (leftTile != null && leftTile.childCount > 0 && leftTile.GetChild(0).GetComponent<PlacementProperties>().objectName == objectName)
        {
            leftTileConnected = true;

            if (updateConnected)
                leftTile.GetChild(0).GetComponent<PlacementProperties>().UpdateOrientation(false);
        }
        if (rightTile != null && rightTile.childCount > 0 && rightTile.GetChild(0).GetComponent<PlacementProperties>().objectName == objectName)
        {
            rightTileConnected = true;

            if (updateConnected)
                rightTile.GetChild(0).GetComponent<PlacementProperties>().UpdateOrientation(false);
        }

        // Change orientation to unconnected
        if (!upTileConnected && !downTileConnected && !leftTileConnected && !rightTileConnected)
        {
            spriteRenderer.sprite = unconnected;
            transform.localEulerAngles = new Vector3(0, 0, 0);
        }
        // Change orientation to connected both sides
        else if (upTileConnected && downTileConnected && !leftTileConnected && !rightTileConnected)
        {
            spriteRenderer.sprite = connectedBothSides;
            transform.localEulerAngles = new Vector3(0, 0, 0);
        }
        else if (!upTileConnected && !downTileConnected && leftTileConnected && rightTileConnected)
        {
            spriteRenderer.sprite = connectedBothSides;
            transform.localEulerAngles = new Vector3(0, 0, -90);
        }
        // Change orientation to connected one side
        else if (upTileConnected && !downTileConnected && !leftTileConnected && !rightTileConnected)
        {
            spriteRenderer.sprite = connectedOneSide;
            transform.localEulerAngles = new Vector3(0, 0, -180);
        }
        else if (!upTileConnected && downTileConnected && !leftTileConnected && !rightTileConnected)
        {
            spriteRenderer.sprite = connectedOneSide;
            transform.localEulerAngles = new Vector3(0, 0, 0);
        }
        else if (!upTileConnected && !downTileConnected && leftTileConnected && !rightTileConnected)
        {
            spriteRenderer.sprite = connectedOneSide;
            transform.localEulerAngles = new Vector3(0, 0, -90);
        }
        else if (!upTileConnected && !downTileConnected && !leftTileConnected && rightTileConnected)
        {
            spriteRenderer.sprite = connectedOneSide;
            transform.localEulerAngles = new Vector3(0, 0, 90);
        }
        // Change orientation to corner
        else if (upTileConnected && !downTileConnected && leftTileConnected && !rightTileConnected)
        {
            spriteRenderer.sprite = corner;
            transform.localEulerAngles = new Vector3(0, 0, -180);
        }
        else if (!upTileConnected && downTileConnected && !leftTileConnected && rightTileConnected)
        {
            spriteRenderer.sprite = corner;
            transform.localEulerAngles = new Vector3(0, 0, 0);
        }
        else if (!upTileConnected && downTileConnected && leftTileConnected && !rightTileConnected)
        {
            spriteRenderer.sprite = corner;
            transform.localEulerAngles = new Vector3(0, 0, -90);
        }
        else if (upTileConnected && !downTileConnected && !leftTileConnected && rightTileConnected)
        {
            spriteRenderer.sprite = corner;
            transform.localEulerAngles = new Vector3(0, 0, 90);
        }
        // Change orientation to t-junction
        else if (upTileConnected && downTileConnected && leftTileConnected && !rightTileConnected)
        {
            spriteRenderer.sprite = tJunction;
            transform.localEulerAngles = new Vector3(0, 0, -180);
        }
        else if (upTileConnected && downTileConnected && !leftTileConnected && rightTileConnected)
        {
            spriteRenderer.sprite = tJunction;
            transform.localEulerAngles = new Vector3(0, 0, 0);
        }
        else if (!upTileConnected && downTileConnected && leftTileConnected && rightTileConnected)
        {
            spriteRenderer.sprite = tJunction;
            transform.localEulerAngles = new Vector3(0, 0, -90);
        }
        else if (upTileConnected && !downTileConnected && leftTileConnected && rightTileConnected)
        {
            spriteRenderer.sprite = tJunction;
            transform.localEulerAngles = new Vector3(0, 0, 90);
        }
        // Change orientation to crossing
        else
            spriteRenderer.sprite = crossing;
    }


    public void DestroyPlacement()
    {
        destroyed = true;
    }
}
