﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class TileSelector : MonoBehaviour
{
    private GameObject placementObject;


    private Gridfield groundGridField;
    public bool canBuild;

    private Vector2 mousePosition = Vector2.zero;
    private Vector2 previousMousePosition = Vector2.zero;

    private GameObject previousTile = null;
    private GameObject currentTile = null;

    public bool isDestroying;

    private void Start()
    {
        groundGridField = GetComponent<Gridfield>();
    }

    private void Update()
    {
        if (canBuild)
        {
            // Keep the mouse position and previous mouse position
            previousMousePosition = mousePosition;
            mousePosition = new Vector2(Camera.main.ScreenToWorldPoint(Input.mousePosition).x, Camera.main.ScreenToWorldPoint(Input.mousePosition).y);

            // Keep the previous tile
            previousTile = currentTile;

            // Only find a new grid when the mouse position has changed
            if (mousePosition != previousMousePosition)
            {
                currentTile = groundGridField.GetTile((int)(mousePosition.x + 0.5f), (int)(mousePosition.y - 0.5f));
            }



            // Highlight tile color when the cursor hovers the tile
            if (currentTile != null)
            {
                // If player is holding an object
                if (placementObject != null)
                {
                    // If tile is buildable
                    if (currentTile.GetComponent<TileInfo>().buildable && currentTile.transform.childCount < 1)
                    {
                        string spriteName = currentTile.GetComponent<SpriteRenderer>().sprite.name;
                        bool canBePlacedOnGrass = placementObject.GetComponent<PlacementProperties>().canBePlacedOnGrass;
                        bool canBePlacedOnSand = placementObject.GetComponent<PlacementProperties>().canBePlacedOnSand;

                        if (spriteName == "Grass" && canBePlacedOnGrass || spriteName == "Sand" && canBePlacedOnSand)
                        {
                            // Show player he can build with a green colour
                            currentTile.GetComponent<SpriteRenderer>().color = Color.green;

                            if (currentTile.transform.childCount == 0 && Input.GetMouseButtonDown(0) && PlayerPrefs.GetInt("Money") >= placementObject.GetComponent<PlacementProperties>().cost)
                            {
                                Instantiate(placementObject, currentTile.transform, false);
                                PlayerPrefs.SetInt("Money", PlayerPrefs.GetInt("Money") - placementObject.GetComponent<PlacementProperties>().cost);
                            }
                        }
                        else
                        {
                            //Show player he can't build with a red colour
                            currentTile.GetComponent<SpriteRenderer>().color = Color.red;
                        }
                    }
                    else
                    {
                        currentTile.GetComponent<SpriteRenderer>().color = Color.red;
                    }

                    // Deselect placement object if players right clicks
                    if (Input.GetMouseButtonDown(1))
                        placementObject = null;
                }
                else
                {
                    currentTile.GetComponent<SpriteRenderer>().color = Color.grey;
                }

                // Deselect placement object if players right clicks
                if (Input.GetMouseButtonDown(1))
                    placementObject = null;

                // Check whether the player destroys a structure
                if (isDestroying && Input.GetMouseButtonDown(0))
                {
                    if (currentTile.transform.childCount > 0 && currentTile.GetComponent<TileInfo>().buildable)
                    {
                        Destroy(currentTile.transform.GetChild(0).gameObject);
                    }

                    DisableDestroying();
                }
            }
            else if (currentTile != null)
            {
                currentTile.GetComponent<SpriteRenderer>().color = Color.grey;
            }

            // Reset tile color when cursor leaves tile
            if (currentTile != previousTile && previousTile != null)
            {
                previousTile.GetComponent<SpriteRenderer>().color = Color.white;
            }
        }
        else if (currentTile != null)
        {
            currentTile.GetComponent<SpriteRenderer>().color = Color.white;
        }
    }

    /// <summary>
    /// Enable object destroying mode
    /// </summary>
    public void EnableDestroying()
    {
        isDestroying = true;

        Deselect();
    }

    /// <summary>
    /// Disable object destroying mode
    /// </summary>
    public void DisableDestroying()
    {
        isDestroying = false;

        Deselect();
    }

    /// <summary>
    /// Sets placement object variable
    /// </summary>
    /// <param name="prefab">Prefab to set placement object to</param>
    public void SetPlacementObject(GameObject prefab)
    {
        if (placementObject != prefab)
            placementObject = prefab;
        else
            placementObject = null;
    }

    public GameObject GetPlacementObject()
    {
        return placementObject;
    }

    /// <summary>
    /// Deselects the placement object
    /// </summary>
    public void Deselect()
    {
        placementObject = null;
    }
}
