﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class WaterIndex : MonoBehaviour {

    public int height, width;

    public GameObject waterPrefab;

    public float waterHeight;
    public float waterMovementSpeed;
    public float drainRate;

    public float offset = 0.25f;
    public int waveStrength;

    private float snapping = 0.5f;

    private GameObject[,] waterTiles;

    private Hashtable tableWaterTiles;

	private void Start()
    {
        waterTiles = new GameObject[width, height];

        for(int x = 0; x < width; x++)
        {
            for(int y = 0; y < height; y++)
            {
                GameObject instance = Instantiate(waterPrefab, new Vector2((x - 0.5f) * snapping, (-y + 0.5f) * snapping), transform.rotation, transform);
                instance.GetComponent<WaterBehaviour>().x = x;
                instance.GetComponent<WaterBehaviour>().y = y;
                instance.GetComponent<WaterBehaviour>().WaterHeight = waterHeight;
                instance.GetComponent<WaterBehaviour>().waterIndex = this;

                waterTiles[x, y] = instance;
            }
        }

        // Populate hashtable
        tableWaterTiles = new Hashtable();

        foreach (GameObject tile in waterTiles)
            tableWaterTiles.Add(new Vector2(tile.transform.position.x, tile.transform.position.y), tile);

        StartCoroutine(UpdateWaterTiles());
    }


    private IEnumerator UpdateWaterTiles()
    {
        yield return new WaitForSeconds(waterMovementSpeed);

        for (int x = 0; x < width; x++)
        {
            for(int y = 0; y < height; y++)
            {
                waterTiles[x, y].GetComponent<WaterBehaviour>().UpdateWater();
            }
        }

        StartCoroutine(UpdateWaterTiles());
    }

    public GameObject GetTile(int x, int y)
    {
        if (x < 0 || x >= width || y < 0 || y >= height)
            return null;
        else
            return waterTiles[x, y];
    }

    public GameObject FindTileByPosition(float x, float y)
    {
        if (tableWaterTiles.Contains(new Vector2(x, y)))
            return (GameObject)tableWaterTiles[new Vector2(x, y)];
        else
            return null;
    }

    public void ActivateWave(float strength)
    {
        // Put water on the side of the game world
        for (int i = 0; i < height; i++)
        {
            waterTiles[0, i].GetComponent<WaterBehaviour>().WaterHeight += strength;
        }

    }

    public void StartTsunami(int beachWidth, float time)
    {
        float startstrength = 10;
        float strength = startstrength;

        drainRate = startstrength / (time / waterMovementSpeed) / 2.5f;
        // Put water on the side of the game world
        for (int i = 0; i < (beachWidth + 1) * 2; i++)
        {
            for (int j = 0; j < height; j++)
            {
                if (waterTiles[i, j] != null)
                {
                    waterTiles[i, j].GetComponent<WaterBehaviour>().WaterHeight += strength;
                    waterTiles[i, j].GetComponent<WaterBehaviour>().flowing = false;
                }
            }
            strength -= startstrength / ((beachWidth + 1) * 2);
        }

    }

    public void ActivateFlowing()
    {
        foreach (GameObject w in waterTiles)
        {
            w.GetComponent<WaterBehaviour>().flowing = true;
        }
    }
}
